//-----------------------------------------------------------------------------
// Author:Jonus				Email: ion_x2@yahoo.com
//
// File: cubeworld.cpp		Date: 17.01.2001
//
// Desc: It shows how u can make a 3d-map out of two int array
//-----------------------------------------------------------------------------

#include <d3dx8.h>
#include <mmsystem.h>

// Our custom FVF, which describes our custom vertex structure
#define D3DFVF_CUSTOMVERTEX (D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_DIFFUSE|D3DFVF_TEX1)


//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
LPDIRECT3D8				g_pD3D			= NULL; // Used to create the D3DDevice
LPDIRECT3DDEVICE8		g_pd3dDevice	= NULL; // Our rendering device
LPDIRECT3DVERTEXBUFFER8	g_pVBMap		= NULL; // Buffer to hold map vertices
LPDIRECT3DTEXTURE8		g_pTilesTexture	= NULL; // Map texture

// A structure for our custom vertex type
struct CUSTOMVERTEX
{
    D3DXVECTOR3 position;	// The untransformed, 3D position for the vertex
    D3DXVECTOR3 normal;   
    D3DCOLOR color;			// The vertex color
	FLOAT tu, tv;			// The texture coordinates
};

typedef struct TEXTURETILES
{
	FLOAT x1,x2,y1,y2;
} t_TexCor;

t_TexCor tex_tile[30];

int map[20][20];
int map_heigth[20][20];

// **********************************************************
void randomise_array(int * p_do_it,int modulo,int multi)
{
for(int i=0;i<20*20;i++)
{
	p_do_it[i] = multi * ((timeGetTime()+rand())%modulo);
}	
}

// Function to create texture coordinates in tex_tile
void createtiles(int x_anz, int y_anz, t_TexCor * destination)
{
	int counttiles=0;
	for(int i=0;i<y_anz;i++){
		for(int j=0;j<x_anz;j++){
			destination[counttiles].y1=(float)i*1/y_anz;
			destination[counttiles].x1=(float)j*1/x_anz;
			destination[counttiles].y2=(float)(1+i)*1/y_anz;
			destination[counttiles].x2=(float)(1+j)*1/x_anz;
			counttiles++;
		}
	}
}

void createsquare(D3DXVECTOR3 D3D_P1,D3DXVECTOR3 D3D_P2,
				  D3DXVECTOR3 D3D_P3,D3DXVECTOR3 D3D_P4,
				  CUSTOMVERTEX * pVer,int buffer_s,int ptiles)
{
	D3DXVECTOR3 temp_sol,temp_1,temp_2;
	temp_1 = D3D_P1 - D3D_P2;
	temp_2 = D3D_P1 - D3D_P3;
	D3DXVec3Cross(&temp_sol, &temp_1, &temp_2);

	pVer[buffer_s+0].position=D3D_P1;
	pVer[buffer_s+0].color=0xffffffff;
	pVer[buffer_s+0].tu= tex_tile[ptiles].x1;
	pVer[buffer_s+0].tv= tex_tile[ptiles].y1;
	pVer[buffer_s+0].normal = temp_sol;
	pVer[buffer_s+1].position=D3D_P3;
	pVer[buffer_s+1].color=0xffffffff;
	pVer[buffer_s+1].tu= tex_tile[ptiles].x2;
	pVer[buffer_s+1].tv= tex_tile[ptiles].y2;
	pVer[buffer_s+1].normal = temp_sol;
	pVer[buffer_s+2].position=D3D_P2;
	pVer[buffer_s+2].color=0xffffffff;
	pVer[buffer_s+2].tu= tex_tile[ptiles].x2;
	pVer[buffer_s+2].tv= tex_tile[ptiles].y1;
	pVer[buffer_s+2].normal = temp_sol;

	pVer[buffer_s+3].position=D3D_P3;
	pVer[buffer_s+3].color=0xffffffff;
	pVer[buffer_s+3].tu= tex_tile[ptiles].x2;
	pVer[buffer_s+3].tv= tex_tile[ptiles].y2;
	pVer[buffer_s+3].normal = temp_sol;
	pVer[buffer_s+4].position=D3D_P1;
	pVer[buffer_s+4].color=0xffffffff;
	pVer[buffer_s+4].tu= tex_tile[ptiles].x1;
	pVer[buffer_s+4].tv= tex_tile[ptiles].y1;
	pVer[buffer_s+4].normal = temp_sol;
	pVer[buffer_s+5].position=D3D_P4;
	pVer[buffer_s+5].color=0xffffffff;
	pVer[buffer_s+5].tu= tex_tile[ptiles].x1;
	pVer[buffer_s+5].tv= tex_tile[ptiles].y2;
	pVer[buffer_s+5].normal = temp_sol;
}

// Function to create cubes
void createcube(float x,float y,float z,
				CUSTOMVERTEX * pVer,int vbuffer, int tex_top, int tex_side)
{
// Top
createsquare(	D3DXVECTOR3(0.0f+x,1.0f+y,0.0f+z),
				D3DXVECTOR3(1.0f+x,1.0f+y,0.0f+z),
				D3DXVECTOR3(1.0f+x,1.0f+y,1.0f+z),
				D3DXVECTOR3(0.0f+x,1.0f+y,1.0f+z),pVer,vbuffer,tex_top);vbuffer+=6;
// The four sides
createsquare(	D3DXVECTOR3(0.0f+x,0.0f+y,0.0f+z),
				D3DXVECTOR3(1.0f+x,0.0f+y,0.0f+z),
				D3DXVECTOR3(1.0f+x,1.0f+y,0.0f+z),
				D3DXVECTOR3(0.0f+x,1.0f+y,0.0f+z),pVer,vbuffer,tex_side);vbuffer+=6;

createsquare(	D3DXVECTOR3(1.0f+x,0.0f+y,0.0f+z),
				D3DXVECTOR3(1.0f+x,0.0f+y,1.0f+z),
				D3DXVECTOR3(1.0f+x,1.0f+y,1.0f+z),
				D3DXVECTOR3(1.0f+x,1.0f+y,0.0f+z),pVer,vbuffer,tex_side);vbuffer+=6;

createsquare(	D3DXVECTOR3(1.0f+x,0.0f+y,1.0f+z),
				D3DXVECTOR3(0.0f+x,0.0f+y,1.0f+z),
				D3DXVECTOR3(0.0f+x,1.0f+y,1.0f+z),
				D3DXVECTOR3(1.0f+x,1.0f+y,1.0f+z),pVer,vbuffer,tex_side);vbuffer+=6;

createsquare(	D3DXVECTOR3(0.0f+x,0.0f+y,1.0f+z),
				D3DXVECTOR3(0.0f+x,0.0f+y,0.0f+z),
				D3DXVECTOR3(0.0f+x,1.0f+y,0.0f+z),
				D3DXVECTOR3(0.0f+x,1.0f+y,1.0f+z),pVer,vbuffer,tex_side);vbuffer+=6;
/* Bottom
createsquare(	D3DXVECTOR3(0.0f+x,0.0f+y,0.0f+z),
				D3DXVECTOR3(1.0f+x,0.0f+y,0.0f+z),
				D3DXVECTOR3(1.0f+x,0.0f+y,1.0f+z),
				D3DXVECTOR3(0.0f+x,0.0f+y,1.0f+z),pVer,vbuffer,tex_side);vbuffer+=6;*/
}

//-----------------------------------------------------------------------------
// Name: InitD3D()
// Desc: Initializes Direct3D
//-----------------------------------------------------------------------------
HRESULT InitD3D( HWND hWnd )
{
    // Create the D3D object.
    if( NULL == ( g_pD3D = Direct3DCreate8( D3D_SDK_VERSION ) ) )
        return E_FAIL;

    // Get the current desktop display mode, so we can set up a back
    // buffer of the same format
    D3DDISPLAYMODE d3ddm;
    if( FAILED( g_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm ) ) )
        return E_FAIL;

    // Set up the structure used to create the D3DDevice
    D3DPRESENT_PARAMETERS d3dpp;
    ZeroMemory( &d3dpp, sizeof(d3dpp) );
    d3dpp.Windowed = TRUE;
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat = d3ddm.Format;
	d3dpp.EnableAutoDepthStencil = TRUE;
	d3dpp.AutoDepthStencilFormat = D3DFMT_D16;

    // Create the D3DDevice
    if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                      &d3dpp, &g_pd3dDevice ) ) )
    {
        return E_FAIL;
    }

    // Turn off culling, so we see the front and back of the triangle
	// g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );

	// alphablend states
	g_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE,   TRUE );
	g_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA );
	g_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );

	// Enable alpha testing (skips pixels with less than a certain alpha.)
	g_pd3dDevice->SetRenderState( D3DRS_ALPHATESTENABLE, TRUE );
	g_pd3dDevice->SetRenderState( D3DRS_ALPHAREF,        0x08 );
	g_pd3dDevice->SetRenderState( D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL );

    // Turn off D3D lighting, since we are providing our own vertex colors
    g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );

    // Turn on the zbuffer
    g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );

    return S_OK;
}


//-----------------------------------------------------------------------------
// Name: InitGeometry()
// Desc: Creates the scene geometry
//-----------------------------------------------------------------------------
HRESULT InitGeometry()
{
	int counter=0;

	randomise_array( *map,10,1);
	randomise_array( *map_heigth,3,1);

	// Texture for Map
	if( FAILED( D3DXCreateTextureFromFileA( g_pd3dDevice, "SF2Tiless.png",&g_pTilesTexture ) ) )
	return E_FAIL;
	createtiles(4,4,tex_tile);

	// Create the vertex buffer. 
	if( FAILED( g_pd3dDevice->CreateVertexBuffer( 5*2*3*20*20*sizeof(CUSTOMVERTEX),
												  0, D3DFVF_CUSTOMVERTEX,
												  D3DPOOL_DEFAULT, &g_pVBMap ) ) )
	{
		return E_FAIL;
	}

	// Fill the vertex buffer
	CUSTOMVERTEX * pmapVertices;
	if( FAILED( g_pVBMap->Lock( 0, 0, (BYTE**)&pmapVertices, 0 ) ) )
		return E_FAIL;

	for(int x_pos=0;x_pos<20;x_pos++)
	{
		for(int z_pos=0;z_pos<20;z_pos++)
		{
		createcube((float)x_pos,(float)map_heigth[x_pos][z_pos]/2,(float)z_pos,
		pmapVertices,counter, map[x_pos][z_pos], map[x_pos][z_pos]);counter+=30;
		}
	}

	g_pVBMap->Unlock();

    return S_OK;
}


//-----------------------------------------------------------------------------
// Name: Cleanup()
// Desc: Releases all previously initialized objects
//-----------------------------------------------------------------------------
VOID Cleanup()
{
	if( g_pTilesTexture != NULL )
		g_pTilesTexture->Release();

    if( g_pVBMap != NULL )
        g_pVBMap->Release();

    if( g_pd3dDevice != NULL )
        g_pd3dDevice->Release();

    if( g_pD3D != NULL )
        g_pD3D->Release();
}



//-----------------------------------------------------------------------------
// Name: SetupMatrices()
// Desc: Sets up the world, view, and projection transform matrices.
//-----------------------------------------------------------------------------
VOID SetupMatrices()
{
    // Here is our world matrix.
    D3DXMATRIX matWorld;
	D3DXMatrixRotationYawPitchRoll( &matWorld, 0.0f,0.0f,0.0f );// <-- Here u can make world transforms
	//D3DXMatrixRotationZ( &matWorld, 0.0f);
    g_pd3dDevice->SetTransform( D3DTS_WORLD, &matWorld );
	
    // Set up our view matrix. A view matrix can be defined given an eye point(1),
    // a point to lookat(2), and a direction for which way is up(3). 
    
    D3DXMATRIX matView;
    D3DXMatrixLookAtLH( &matView, &D3DXVECTOR3( 10.0f+5*cosf(timeGetTime()/3000.0f), 7.0f+cosf(timeGetTime()/3000.0f), 10.0f+5*sinf(timeGetTime()/4000.0f) ),// (1)
                                  &D3DXVECTOR3( 10.5f +4*sinf(timeGetTime()/2000.0f), 0.0f, 10.5f+sinf(timeGetTime()/2000.0f) ),// (2) 
                                  &D3DXVECTOR3( 0.0f, 1.0f, 0.0f ) );// (3)
    g_pd3dDevice->SetTransform( D3DTS_VIEW, &matView );

    // For the projection matrix, we set up a perspective transform (which
    // transforms geometry from 3D view space to 2D viewport space, with
    // a perspective divide making objects smaller in the distance). To build
    // a perpsective transform, we need the field of view (1/4 pi is common),
    // the aspect ratio, and the near and far clipping planes (which define at
    // what distances geometry should be no longer be rendered).
    D3DXMATRIX matProj;
    D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, 1.5f, 1.0f, 100.0f );
	//D3DXMatrixPerspectiveLH( &matProj, 1.0f, 1.0f, 1.0f, 100.0f );

    g_pd3dDevice->SetTransform( D3DTS_PROJECTION, &matProj );
}




//-----------------------------------------------------------------------------
// Name: SetupLights()
// Desc: Sets up the lights and materials for the scene.
//-----------------------------------------------------------------------------
VOID SetupLights()
{
    // Set up a material. The material here just has the diffuse and ambient
    // colors set to yellow. Note that only one material can be used at a time.
    D3DMATERIAL8 mtrl;
    ZeroMemory( &mtrl, sizeof(D3DMATERIAL8) );
    mtrl.Diffuse.r = mtrl.Ambient.r = 1.0f;
    mtrl.Diffuse.g = mtrl.Ambient.g = 1.0f;
    mtrl.Diffuse.b = mtrl.Ambient.b = 1.0f;
    mtrl.Diffuse.a = mtrl.Ambient.a = 1.0f;
    g_pd3dDevice->SetMaterial( &mtrl );

    // Set up a white, directional light, with an oscillating direction.
    // Note that many lights may be active at a time (but each one slows down
    // the rendering of our scene). However, here we are just using one. Also,
    // we need to set the D3DRS_LIGHTING renderstate to enable lighting
    D3DXVECTOR3 vecDir;
    D3DLIGHT8 light;
    ZeroMemory( &light, sizeof(D3DLIGHT8) );
    light.Type       = D3DLIGHT_DIRECTIONAL;
	float temp_color = (float) timeGetTime();
    light.Diffuse.r  = (float)0.74f+cosf(temp_color/10000)*0.25f;
    light.Diffuse.g  = 1.0f;
    light.Diffuse.b  = (float)0.9f+cosf(temp_color/20000)*0.1f;
    vecDir = D3DXVECTOR3(10.0f+14.0f*cosf(timeGetTime()/3000.0f),
                         10.0f+ 5.0f*cosf(timeGetTime()/10000.0f),
                         10.0f+14.0f*sinf(timeGetTime()/3500.0f) );
    D3DXVec3Normalize( (D3DXVECTOR3*)&light.Direction, &vecDir );
    light.Range       = 1000.0f;
    g_pd3dDevice->SetLight( 0, &light );
    g_pd3dDevice->LightEnable( 0, TRUE );
    g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE );

    // Finally, turn on some ambient light.
    g_pd3dDevice->SetRenderState( D3DRS_AMBIENT, 0x00202020 );
}



//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Draws the scene
//-----------------------------------------------------------------------------
VOID Render()
{
    // Clear the backbuffer and the zbuffer
    g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
                         D3DCOLOR_XRGB(95,125,255), 1.0f, 0 );

    // Begin the scene
    g_pd3dDevice->BeginScene();

    // Setup the lights and materials
    SetupLights();

    // Setup the world, view, and projection matrices
    SetupMatrices();


    // Setup our texture
    g_pd3dDevice->SetTexture( 0, g_pTilesTexture ); // Now use TilesTexture
    g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
    g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
    g_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1 );

    // Render the map vertex buffer contents 
    g_pd3dDevice->SetStreamSource( 0, g_pVBMap, sizeof(CUSTOMVERTEX) );
    g_pd3dDevice->SetVertexShader( D3DFVF_CUSTOMVERTEX );
    g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 5*2*20*20 );


    // End the scene
    g_pd3dDevice->EndScene();

    // Present the backbuffer contents to the display
    g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
}




//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: The window's message handler
//-----------------------------------------------------------------------------
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
        case WM_DESTROY:
            PostQuitMessage( 0 );
            return 0;
    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}


//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: The application's entry point
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    // Register the window class
    WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L,
                      GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
                      "D3D Tutorial", NULL };
    RegisterClassEx( &wc );

    // Create the application's window
    HWND hWnd = CreateWindow( "D3D Tutorial", "D3D Tutorial 13: Cubeworld",
                              WS_OVERLAPPEDWINDOW, 100, 100, 400, 300,
                              GetDesktopWindow(), NULL, wc.hInstance, NULL );

    // Initialize Direct3D
    if( SUCCEEDED( InitD3D( hWnd ) ) )
    {
        // Create the scene geometry
        if( SUCCEEDED( InitGeometry() ) )
        {
            // Show the window
            ShowWindow( hWnd, SW_SHOWDEFAULT );
            UpdateWindow( hWnd );

            // Enter the message loop
            MSG msg;
            ZeroMemory( &msg, sizeof(msg) );
            while( msg.message!=WM_QUIT )
            {
                if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
                {
                    TranslateMessage( &msg );
                    DispatchMessage( &msg );
                }
                else
					// Here are the Gameloops :)
                    Render();
            }
        }
    }

    // Clean up everything and exit the app
    Cleanup();
    UnregisterClass( "D3D Tutorial", wc.hInstance );
    return 0;
}



